@@ -1,3 +1,4 @@
+/* vim: set sw=8: -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 #include <config.h>
 #include "gnome-cups-request.h"
 #include <glib.h>
@@ -55,6 +56,8 @@ static void gnome_cups_request_connection_destroy (GnomeCupsConnection *conn);
 static gboolean idle_stop_unused_threads (gpointer unused);
 static gboolean idle_close_unused_connections (gpointer unused);
 
+gboolean _gnome_cups_debug = FALSE;
+
 static const char *
 get_error_string (ipp_status_t error)
 {  
@@ -170,6 +173,9 @@ _gnome_cups_request_init (GnomeCupsAuthFunction auth_fn)
 	GError *error = NULL;
 	global_auth = auth_fn;
 	cupsSetPasswordCB (cups_password_cb);
+
+	if (!g_thread_supported ())
+		g_thread_init (NULL);
 	
 	g_static_mutex_lock (&request_mutex);
 	if (request_system_refcount == 0) {
@@ -321,25 +327,40 @@ do_signal_complete (GnomeCupsRequest *request)
 
 static void
 request_thread_main (GnomeCupsRequest *request,
-		     gpointer unused)
+		     G_GNUC_UNUSED gpointer unused)
 {
 	ipp_t *response;
 	ipp_status_t status;
+	unsigned i;
+
+	g_return_if_fail (request != NULL);
 
 	if (request->cancelled) {
 		do_signal_complete (request);
 		return;
 	}
 
+	if (_gnome_cups_debug) {
+		g_print ("---->>>  locking %p\n", request->connection);
+	}
 	g_mutex_lock (request->connection->mutex);
 	
+	if (_gnome_cups_debug && request->request != NULL) {
+		ipp_attribute_t *attr;
+		for (attr = request->request->attrs; attr != NULL; attr = attr->next) {
+			g_print ("%s", attr->name);
+			for (i = 0 ; i < attr->num_values ; i++)
+				if (attr->value_tag >= IPP_TAG_STRING)
+				    g_print ("\t[%d] = '%s'\n", i, attr->values[i].string.text);
+		}
+	}
+
 	g_get_current_time (&request->connection->use_time);
 
 	/* This is a deferred open for the first connection */
 	if (!request->connection->http)
 		request->connection->http = httpConnectEncrypt (request->connection->server, ippPort(), cupsEncryption());
 
-
 	response = cupsDoRequest (request->connection->http, request->request,
 				  request->path);
 
@@ -353,6 +374,9 @@ request_thread_main (GnomeCupsRequest *request,
 
 	g_atomic_int_dec_and_test (&request->connection->refcount);
 	g_mutex_unlock (request->connection->mutex);
+	if (_gnome_cups_debug) {
+		g_print ("<<<<----- unlocking %p\n", request->connection);
+	}
 
 	if (status > IPP_OK_CONFLICT && request->error != NULL) {
 		*(request->error) = g_error_new (GNOME_CUPS_ERROR, 
